==> beaker_kernel-1.4.5.tar.gz: beaker_kernel-1.4.5/beaker_kernel/server/main.py <==
import datetime
import json
import logging
import os
from typing import Dict

from jupyter_core.utils import ensure_async
from jupyter_server.auth.decorator import authorized
from jupyter_server.base.handlers import JupyterHandler
from jupyter_server.extension.handler import ExtensionHandlerMixin
from jupyter_server.services.kernels.handlers import MainKernelHandler, json_default
from jupyter_server.utils import url_escape, url_path_join
from jupyterlab_server import LabServerApp
from tornado import web
from tornado.web import StaticFileHandler, RedirectHandler, RequestHandler, HTTPError

from beaker_kernel.lib.autodiscovery import autodiscover
from beaker_kernel.lib.context import BaseContext
from beaker_kernel.lib.subkernels.base import BaseSubkernel

logger = logging.getLogger(__file__)

HERE = os.path.dirname(__file__)

version = "0.0.1"


def _jupyter_server_extension_points():
    return [{"module": "beaker_kernel.server.main", "app": BeakerJupyterApp}]


def secure_env(env: dict) -> dict:
        UNSAFE_WORDS = ["KEY", "SECRET", "TOKEN", "PASSWORD"]
        safe_env = {}
        for env_name, env_value in env.items():
            for unsafe_word in UNSAFE_WORDS:
                if unsafe_word in env_name.upper():
                    break
            else:
                safe_env[env_name] = env_value
        return safe_env


class SafeKernelHandler(MainKernelHandler):

    @web.authenticated
    @authorized
    async def post(self):
        km = self.kernel_manager
        model = self.get_json_body()
        if model is None:
            model = {"name": km.default_kernel_name}
        else:
            model.setdefault("name", km.default_kernel_name)

        env = self.session_manager.get_kernel_env(path=model.get("path"), name=model["name"])
        env = secure_env(env)

        kernel_id = await ensure_async(
            km.start_kernel(  # type:ignore[has-type]
                kernel_name=model["name"], path=model.get("path"), env=env
            )
        )
        model = await ensure_async(km.kernel_model(kernel_id))
        location = url_path_join(self.base_url, "api", "kernels", url_escape(kernel_id))
        self.set_header("Location", location)
        self.set_status(201)
        self.finish(json.dumps(model, default=json_default))


class NotebookHandler(ExtensionHandlerMixin, JupyterHandler):

    def get(self):
        return self.write(json.dumps(notebook_content))

    def post(self):
        global notebook_content
        notebook_content = self.get_json_body()
        notebook_content["lastSaved"] = datetime.datetime.utcnow().isoformat()
        return self.write(json.dumps(notebook_content))


class ConfigHandler(ExtensionHandlerMixin, JupyterHandler):
    """
    Provide config via an endpoint
    """

    def get(self):
        """
        """
        base_url = f"{self.request.protocol}://{self.request.host}"
        if self.request.protocol.endswith("s"):
            ws_proto = "wss"
        else:
            ws_proto = "ws"
        ws_url = f"{ws_proto}://{self.request.host}"

        config_data = {
            "appUrl": os.environ.get("APP_URL", base_url),
            "baseUrl": os.environ.get("JUPYTER_BASE_URL", base_url),
            "wsUrl": os.environ.get("JUPYTER_WS_URL", ws_url),
            "token": os.environ.get("JUPYTER_TOKEN", "89f73481102c46c0bc13b2998f9a4fce"),
        }
        return self.write(config_data)


class ContextHandler(ExtensionHandlerMixin, JupyterHandler):
    """
    Provide information about llm contexts via an endpoint
    """

    def get(self):
        """Get the main page for the application's interface."""
        ksm = self.kernel_spec_manager
        contexts: Dict[str, BaseContext] = autodiscover("contexts")
        possible_subkernels: Dict[str, BaseSubkernel] = autodiscover("subkernels")
        subkernel_by_kernel_index = {subkernel.KERNEL_NAME: subkernel for subkernel in possible_subkernels.values()}
        installed_kernels = [
            subkernel_by_kernel_index[kernel_name] for kernel_name in ksm.find_kernel_specs().keys()
            if kernel_name in subkernel_by_kernel_index
        ]
        contexts = sorted(contexts.items(), key=lambda item: (item[1].WEIGHT, item[0]))

        # Extract data from auto-discovered contexts and subkernels to provide options
        context_data = {
            context_slug: {
                "languages": [
                    {
                        "slug": subkernel_slug,
                        "subkernel": getattr(possible_subkernels.get(subkernel_slug), "KERNEL_NAME")
                    }
                    for subkernel_slug in context.available_subkernels()
                    if subkernel_slug in set(subkernel.SLUG for subkernel in installed_kernels)
                ],
                "defaultPayload": context.default_payload()
            }
            for context_slug, context in contexts
        }
        return self.write(context_data)


class UploadHandler(RequestHandler):
    def post(self):
        filenames = []
        for file in self.request.files["uploadfiles"]:
            # Files are actually an array within an array?
            filename = file["filename"]
            if os.path.exists(filename):
                self.write(f"'{filename}' already exists.")
                raise HTTPError(401)
            with open(filename, 'wb') as output_file:
                output_file.write(file["body"])
            filenames.append(filename)
        self.finish(f"Saved files {', '.join(filenames)} uploaded.")


class DownloadHandler(RequestHandler):
    def get(self, filepath: str):
        if filepath.startswith("."):
            raise HTTPError(404)
        if os.path.isfile(filepath):
            _, file_name = os.path.split(filepath)
            if file_name.startswith("."):
                raise HTTPError(404)

            self.set_header('Content-Type', 'application/force-download')
            self.set_header('Content-Disposition', f'attachment; filename=%s' % file_name)
            try:
                with open(filepath, "rb") as f:
                    while True:
                        _buffer = f.read(4096)
                        if _buffer:
                            self.write(_buffer)
                        else:
                            f.close()
                            self.finish()
                            return
            except:
                raise HTTPError(404)
        else:
            raise HTTPError(404)


class BeakerJupyterApp(LabServerApp):
    name = "beaker_kernel"
    load_other_extensions = True
    app_name = "Beaker Jupyter App"
    app_version = version
    allow_origin = "*"
    open_browser = False
    extension_url = "/"

    def initialize_handlers(self):
        """Bypass initializing the default handler since we don't need to use the webserver, just the websockets."""
        self.handlers.append((r"/api/kernels", SafeKernelHandler))
        self.handlers.append(("/contexts", ContextHandler))
        self.handlers.append(("/config", ConfigHandler))
        self.handlers.append(("/notebook", NotebookHandler))
        self.handlers.append((r"/upload", UploadHandler))
        self.handlers.append((r"/download/(.*)", DownloadHandler))
        self.handlers.append((r"(/?)", StaticFileHandler, {"path": os.path.join(HERE, "ui"), "default_filename": "index.html"}))
        self.handlers.append((r"/index.html", StaticFileHandler, {"path": os.path.join(HERE, "ui"), "default_filename": "index.html"}))
        self.handlers.append((r"/(favicon.ico)", StaticFileHandler, {"path": os.path.join(HERE, "ui")}))
        self.handlers.append((r"/static/(.*)", StaticFileHandler, {"path": os.path.join(HERE, "ui", "static")}))
        self.handlers.append((r"/themes/(.*)", StaticFileHandler, {"path": os.path.join(HERE, "ui", "themes")}))
        self.handlers.append((r"/dev_ui/?(.*)", RedirectHandler, {"url": r"/{0}"}))
        super().initialize_handlers()

    def initialize_settings(self):
        # Override to allow cross domain websockets
        self.settings["allow_origin"] = "*"
        self.settings["disable_check_xsrf"] = True


if __name__ == "__main__":
    BeakerJupyterApp.launch_instance()

==> beaker_kernel-1.4.8.tar.gz: beaker_kernel-1.4.8/beaker_kernel/server/main.py <==
import datetime
import json
import logging
import os
import uuid
from typing import Dict

from jupyter_core.utils import ensure_async
from jupyter_server.auth.decorator import authorized
from jupyter_server.base.handlers import JupyterHandler
from jupyter_server.extension.handler import ExtensionHandlerMixin
from jupyter_server.services.kernels.handlers import MainKernelHandler, json_default
from jupyter_server.utils import url_escape, url_path_join
from jupyterlab_server import LabServerApp
from tornado import web, httputil
from tornado.web import StaticFileHandler, RedirectHandler, RequestHandler, HTTPError

from beaker_kernel.lib.autodiscovery import autodiscover
from beaker_kernel.lib.context import BaseContext
from beaker_kernel.lib.subkernels.base import BaseSubkernel

logger = logging.getLogger(__file__)

HERE = os.path.dirname(__file__)

version = "0.0.1"


def _jupyter_server_extension_points():
    return [{"module": "beaker_kernel.server.main", "app": BeakerJupyterApp}]


def secure_env(env: dict) -> dict:
        UNSAFE_WORDS = ["KEY", "SECRET", "TOKEN", "PASSWORD"]
        safe_env = {}
        for env_name, env_value in env.items():
            for unsafe_word in UNSAFE_WORDS:
                if unsafe_word in env_name.upper():
                    break
            else:
                safe_env[env_name] = env_value
        return safe_env


class SafeKernelHandler(MainKernelHandler):

    @web.authenticated
    @authorized
    async def post(self):
        km = self.kernel_manager
        model = self.get_json_body()
        if model is None:
            model = {"name": km.default_kernel_name}
        else:
            model.setdefault("name", km.default_kernel_name)

        env = self.session_manager.get_kernel_env(path=model.get("path"), name=model["name"])
        env = secure_env(env)

        kernel_id = await ensure_async(
            km.start_kernel(  # type:ignore[has-type]
                kernel_name=model["name"], path=model.get("path"), env=env
            )
        )
        model = await ensure_async(km.kernel_model(kernel_id))
        location = url_path_join(self.base_url, "api", "kernels", url_escape(kernel_id))
        self.set_header("Location", location)
        self.set_status(201)
        self.finish(json.dumps(model, default=json_default))


class NotebookHandler(ExtensionHandlerMixin, JupyterHandler):

    def get(self):
        return self.write(json.dumps(notebook_content))

    def post(self):
        global notebook_content
        notebook_content = self.get_json_body()
        notebook_content["lastSaved"] = datetime.datetime.utcnow().isoformat()
        return self.write(json.dumps(notebook_content))


class MainHandler(StaticFileHandler):
    """
    Handle the main interface to properly set a session
    """
    async def get(self, path: str, include_body: bool = True) -> None:
        # If no session is provided on a root request, generate a session uuid and redirect to it
        if path == "":
            session_id = self.get_query_argument("session", None)
            if not session_id:
                session_id = str(uuid.uuid4())
                to_url = httputil.url_concat(
                    path,
                    {"session": session_id},
                )
                return self.redirect(to_url, permanent=False)
        # Otherwise, serve files as normal
        return await super().get(path, include_body=include_body)


class ConfigHandler(ExtensionHandlerMixin, JupyterHandler):
    """
    Provide config via an endpoint
    """

    def get(self):
        """
        """
        base_url = f"{self.request.protocol}://{self.request.host}"
        if self.request.protocol.endswith("s"):
            ws_proto = "wss"
        else:
            ws_proto = "ws"
        ws_url = f"{ws_proto}://{self.request.host}"

        config_data = {
            "appUrl": os.environ.get("APP_URL", base_url),
            "baseUrl": os.environ.get("JUPYTER_BASE_URL", base_url),
            "wsUrl": os.environ.get("JUPYTER_WS_URL", ws_url),
            "token": os.environ.get("JUPYTER_TOKEN", "89f73481102c46c0bc13b2998f9a4fce"),
        }
        return self.write(config_data)


class ContextHandler(ExtensionHandlerMixin, JupyterHandler):
    """
    Provide information about llm contexts via an endpoint
    """

    def get(self):
        """Get the main page for the application's interface."""
        ksm = self.kernel_spec_manager
        contexts: Dict[str, BaseContext] = autodiscover("contexts")
        possible_subkernels: Dict[str, BaseSubkernel] = autodiscover("subkernels")
        subkernel_by_kernel_index = {subkernel.KERNEL_NAME: subkernel for subkernel in possible_subkernels.values()}
        installed_kernels = [
            subkernel_by_kernel_index[kernel_name] for kernel_name in ksm.find_kernel_specs().keys()
            if kernel_name in subkernel_by_kernel_index
        ]
        contexts = sorted(contexts.items(), key=lambda item: (item[1].WEIGHT, item[0]))

        # Extract data from auto-discovered contexts and subkernels to provide options
        context_data = {
            context_slug: {
                "languages": [
                    {
                        "slug": subkernel_slug,
                        "subkernel": getattr(possible_subkernels.get(subkernel_slug), "KERNEL_NAME")
                    }
                    for subkernel_slug in context.available_subkernels()
                    if subkernel_slug in set(subkernel.SLUG for subkernel in installed_kernels)
                ],
                "defaultPayload": context.default_payload()
            }
            for context_slug, context in contexts
        }
        return self.write(context_data)


class UploadHandler(RequestHandler):
    def post(self):
        filenames = []
        for file in self.request.files["uploadfiles"]:
            # Files are actually an array within an array?
            filename = file["filename"]
            if os.path.exists(filename):
                self.write(f"'{filename}' already exists.")
                raise HTTPError(401)
            with open(filename, 'wb') as output_file:
                output_file.write(file["body"])
            filenames.append(filename)
        self.finish(f"Saved files {', '.join(filenames)} uploaded.")


class DownloadHandler(RequestHandler):
    def get(self, filepath: str):
        if filepath.startswith("."):
            raise HTTPError(404)
        if os.path.isfile(filepath):
            _, file_name = os.path.split(filepath)
            if file_name.startswith("."):
                raise HTTPError(404)

            self.set_header('Content-Type', 'application/force-download')
            self.set_header('Content-Disposition', f'attachment; filename=%s' % file_name)
            try:
                with open(filepath, "rb") as f:
                    while True:
                        _buffer = f.read(4096)
                        if _buffer:
                            self.write(_buffer)
                        else:
                            f.close()
                            self.finish()
                            return
            except:
                raise HTTPError(404)
        else:
            raise HTTPError(404)


class BeakerJupyterApp(LabServerApp):
    name = "beaker_kernel"
    load_other_extensions = True
    app_name = "Beaker Jupyter App"
    app_version = version
    allow_origin = "*"
    open_browser = False
    extension_url = "/"

    def initialize_handlers(self):
        """Bypass initializing the default handler since we don't need to use the webserver, just the websockets."""
        self.handlers.append((r"/api/kernels", SafeKernelHandler))
        self.handlers.append(("/contexts", ContextHandler))
        self.handlers.append(("/config", ConfigHandler))
        self.handlers.append(("/notebook", NotebookHandler))
        self.handlers.append((r"/upload", UploadHandler))
        self.handlers.append((r"/download/(.*)", DownloadHandler))
        self.handlers.append((r"(/?)", MainHandler, {"path": os.path.join(HERE, "ui"), "default_filename": "index.html"}))
        self.handlers.append((r"/index.html", StaticFileHandler, {"path": os.path.join(HERE, "ui"), "default_filename": "index.html"}))
        self.handlers.append((r"/(favicon.ico)", StaticFileHandler, {"path": os.path.join(HERE, "ui")}))
        self.handlers.append((r"/static/(.*)", StaticFileHandler, {"path": os.path.join(HERE, "ui", "static")}))
        self.handlers.append((r"/themes/(.*)", StaticFileHandler, {"path": os.path.join(HERE, "ui", "themes")}))
        self.handlers.append((r"/dev_ui/?(.*)", RedirectHandler, {"url": r"/{0}"}))
        super().initialize_handlers()

    def initialize_settings(self):
        # Override to allow cross domain websockets
        self.settings["allow_origin"] = "*"
        self.settings["disable_check_xsrf"] = True


if __name__ == "__main__":
    BeakerJupyterApp.launch_instance()
